
// DEMO13_11.CPP - Time based motion modeling
// to compile make sure to include DDRAW.LIB, DSOUND.LIB,
// DINPUT.LIB, WINMM.LIB, and of course the T3DLIB files

// INCLUDES ///////////////////////////////////////////////

#define INITGUID

#define WIN32_LEAN_AND_MEAN  

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <iostream.h> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h> 
#include <math.h>
#include <io.h>
#include <fcntl.h>

#include <ddraw.h>  // directX includes
#include <dsound.h>
#include <dmksctrl.h>
#include <dmusici.h>
#include <dmusicc.h>
#include <dmusicf.h>
#include <dinput.h>
#include "T3DLIB1.h" // game library includes
#include "T3DLIB2.h"
#include "T3DLIB3.h"

// DEFINES ////////////////////////////////////////////////

// defines for windows 
#define WINDOW_CLASS_NAME "WINXCLASS"  // class name

#define WINDOW_WIDTH    320  // size of window
#define WINDOW_HEIGHT   240

// variable indices
#define SHIP_X_POS  0
#define SHIP_Y_POS  1
#define SHIP_X_VEL  2
#define SHIP_STATE_OFF        0   // test if ship explodes
#define SHIP_STATE_ON         1   // test if ship explodes

// variable indices
#define ROCK_X_POS  0
#define ROCK_Y_POS  1
#define ROCK_X_VEL  2

#define ROCK_LARGE            0   // sizes of rock
#define ROCK_MEDIUM           1
#define ROCK_SMALL            2

#define MAX_ROCKS             100
#define ROCK_STATE_OFF        0   // this rock is dead or off
#define ROCK_STATE_ON         1   // this one is alive and in flight

// explosion defines 
#define MAX_BURSTS            8
#define BURST_STATE_OFF       0   // this burst is dead or off
#define BURST_STATE_ON        1   // this one is alive
#define BURST_ROCK        0   // this one is alive
#define BURST_PLAYER        1   // this one is alive

// variable indices
#define ENEMY_X_POS  0
#define ENEMY_Y_POS  1
#define ENEMY_X_VEL  2

#define MAX_ENEMY             1
#define ENEMY_STATE_OFF        0   // this rock is dead or off
#define ENEMY_STATE_ON         1   // this one is alive and in flight



int rock_sizes[3] = {96,56,32}; // X,Y dims for scaler
int ROCKS_OUT=0;
int CRASH=0;
int FRAME_TIMER=99;				//Timer to Keep Action Going While Pausing Another Task
int SCORE=0;
int PLAYER_LIVES=3;
int LEVEL=1;
int ROCKS_DESTROYED=0;
int TOTAL_ROCKS_DESTROYED=0;
int ROCKS_PASSED=0;
int ROCK_OFFSET=99;
int SPEED_OFFSET=1;
int Speed=3;
int speed_lock=0;

#define SHIP_INITIAL_VELOCITY   0.050    // 50 pixels/sec

// defines for the star field



#define MAX_STARS                256   // number of stars in universe

#define STAR_PLANE_0             0    // far plane
#define STAR_PLANE_1             1    // near plane
#define STAR_PLANE_2             2

#define STAR_COLOR_0             8    // color of farthest star plane
#define STAR_COLOR_1             7
#define STAR_COLOR_2             15   // color of nearest star plane


// weapons defines
#define INDEX_WORLD_X		8
#define INDEX_WORLD_Y		9
#define MAX_PLASMA         32   // max number of plasma pulses
#define PLASMA_SPEED       16   // pixels/sec of plasma pulse
#define PLASMA_SPEED_SLOW  12   // slow version for gunships

#define PLASMA_STATE_OFF   0   // this plasma is dead or off
#define PLASMA_STATE_ON    1   // this one is alive and in flight
#define PLASMA_RANGE_1     100  // type 1 plasma disruptor

#define PLASMA_ANIM_PLAYER     0  // a plasma torpedeo from player
#define PLASMA_ANIM_ENEMY      1  // a plasma torpedeo from any enemy

PALETTEENTRY game_palette[256]; // holds the main game palette
PALETTEENTRY ship_palette[256]; // holds the main game palette


// MACROS ///////////////////////////////////////////////

#define RAND_RANGE(x,y) ( (x) + (rand()%((y)-(x)+1)))
#define DOT_PRODUCT(ux,uy,vx,vy) ((ux)*(vx) + (uy)*(vy))

// PROTOTYPES /////////////////////////////////////////////

// game console
int Game_Init(void *parms=NULL);
int Game_Shutdown(void *parms=NULL);
int Game_Main(void *parms=NULL);
void Start_Burst(int x, int y, int width, int height, int xv,int yv, int burst_type);
void Level_Increment();
void Game_Over();
void Refresh_Rocks();
int Copy_Screen(UCHAR *source_bitmap,UCHAR *dest_buffer, int lpitch, int transparent);


inline void Copy_Palette(LPPALETTEENTRY dest, LPPALETTEENTRY source)
{ memcpy(dest, source, 256*sizeof(PALETTEENTRY)); }

// TYPES /////
// used to contain a single star
typedef struct STAR_TYP
        {
        int x,y;                   // position of star
        UCHAR color;               // color of star
        int plane;                 // plane that star is in

        } STAR, *STAR_PTR;



// GLOBALS ////////////////////////////////////////////////

HWND main_window_handle   = NULL; // save the window handle
HINSTANCE main_instance   = NULL; // save the instance
char buffer[256];                 // used to print text

BITMAP_IMAGE background_bmp;      // holds the background
BOB          ship;                // the ship
BOB         rocks[MAX_ROCKS];               // the rock (test)
BOB rock_s, rock_m, rock_l;   // the master templates for surface info
BOB plasma[MAX_PLASMA];       // plasma pulses
BOB bursts[MAX_BURSTS];       // the explosion bursts

int engine_id = -1;              // sound of engine
int laser_id = -1;              // sound of engine
int explode_id = -1;              // sound of explosion
int background_id[5];              // in game music

STAR stars[MAX_STARS];          // the star field

// positional and state info for player
float player_x=0,
      player_y=0,
	  player_dx=0, 
      player_dy=0, 
	  player_xv=0, 
	  player_yv=0;

// FUNCTIONS //////////////////////////////////////////////

LRESULT CALLBACK WindowProc(HWND hwnd, 
						    UINT msg, 
                            WPARAM wparam, 
                            LPARAM lparam)
{
// this is the main message handler of the system
PAINTSTRUCT	ps;		   // used in WM_PAINT
HDC			hdc;	   // handle to a device context

// what is the message 
switch(msg)
	{	
	case WM_CREATE: 
        {
		// do initialization stuff here
		return(0);
		} break;

    case WM_PAINT:
         {
         // start painting
         hdc = BeginPaint(hwnd,&ps);

         // end painting
         EndPaint(hwnd,&ps);
         return(0);
        } break;

	case WM_DESTROY: 
		{
		// kill the application			
		PostQuitMessage(0);
		return(0);
		} break;

	default:break;

    } // end switch

// process any messages that we didn't take care of 
return (DefWindowProc(hwnd, msg, wparam, lparam));

} // end WinProc


void Level_Increment()
{
LEVEL++;

if(LEVEL%5==0)
{DMusic_Play(background_id[(RAND_RANGE(0,4))]);}

if(LEVEL%2==0)
{ROCK_OFFSET--;

}

else
{SPEED_OFFSET++;
}

}

void Game_Over()
{
ship.state=SHIP_STATE_OFF;

for (int i=0;i<MAX_ROCKS;i++)
{rocks[i].state=ROCK_STATE_OFF;}

}

void Game_Restart()
{
PLAYER_LIVES=3;

ship.state=SHIP_STATE_ON;

for (int i=0;i<MAX_ROCKS;i++)
{rocks[i].state=ROCK_STATE_ON;}

ROCKS_OUT=0;
CRASH=0;
FRAME_TIMER=99;				//Timer to Keep Action Going While Pausing Another Task
SCORE=0;
PLAYER_LIVES=3;
LEVEL=1;
ROCKS_DESTROYED=0;
TOTAL_ROCKS_DESTROYED=0;
ROCKS_PASSED=0;
ROCK_OFFSET=99;
SPEED_OFFSET=1;
Speed=3;
speed_lock=0;

Wait_Clock(1000);

ship.varsF[SHIP_X_POS] = 0;
ship.varsF[SHIP_Y_POS] = screen_height/2;

Refresh_Rocks();
DMusic_Play(background_id[(RAND_RANGE(0,4))]);

}

///////////////////////////////////////////////////////////

int Copy_Screen(UCHAR *source_bitmap,UCHAR *dest_buffer, int lpitch, int transparent)
{
// this function draws the bitmap onto the destination memory surface
// if transparent is 1 then color 0 will be transparent
// note this function does NOT clip, so be carefull!!!

UCHAR *dest_addr,   // starting address of bitmap in destination
      *source_addr; // starting adddress of bitmap data in source

UCHAR pixel;        // used to hold pixel value

int index,          // looping vars
    pixel_x;

// test if this bitmap is loaded

   // compute starting destination address
   dest_addr = dest_buffer;

   // compute the starting source address
   source_addr = source_bitmap;

   // is this bitmap transparent
   if (transparent)
   {
   // copy each line of bitmap into destination with transparency
   for (index=0; index < SCREEN_HEIGHT; index++)
       {
       // copy the memory
       for (pixel_x=0; pixel_x < SCREEN_WIDTH; pixel_x++)
           {
           if ((pixel = source_addr[pixel_x])!=0)
               dest_addr[pixel_x] = pixel;

           } // end if

       // advance all the pointers
       dest_addr   += lpitch;
       source_addr += SCREEN_WIDTH;

       } // end for index
   } // end if
   else
   {
   // non-transparent version
   // copy each line of bitmap into destination

   for (index=0; index < SCREEN_HEIGHT; index++)
       {
       // copy the memory
       memcpy(dest_addr, source_addr, SCREEN_WIDTH);

       // advance all the pointers
       dest_addr   += lpitch;
       source_addr += SCREEN_WIDTH;

       } // end for index

   } // end else

   // return success
   return(1);

} // end Copy_Screen

/////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////

int Pad_Name(char *filename, char *extension, char *padstring, int num)
{
int index;
char buffer[80];

// build up blank padstring
sprintf(padstring, "%s0000.%s", filename, extension);

// this function pads a string with 0's
itoa(num, buffer, 10);

// compute the position of the last digit
int last_digit = strlen(filename)+4-1;

// now copy the number into the padstring at the correct position
memcpy(padstring + last_digit - strlen(buffer) + 1, buffer, strlen(buffer));

// return success
return(1);

} // end Pad_Name

///////////////////////////////////////////////////////////

void Init_Ship(void)
{

// load the bitmaps
Load_Bitmap_File(&bitmap8bit, "gunship.bmp");

// create ship
Create_BOB(&ship,0,0,80,32,1,BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, DDSCAPS_SYSTEMMEMORY);

// load the imagery in
Load_Frame_BOB(&ship, &bitmap8bit, 0, 0,0,BITMAP_EXTRACT_MODE_CELL);

// unload bitmap image
Unload_Bitmap_File(&bitmap8bit);

// use element 0,1 to hold float accurate position
ship.varsF[SHIP_X_POS] = 0;
ship.varsF[SHIP_Y_POS] = screen_height/2;

// use index 2 to hold x velocity
ship.varsF[SHIP_X_VEL] = SHIP_INITIAL_VELOCITY; 

//ship is up and running
ship.state=SHIP_STATE_ON;

}

void Draw_Ship(void)
{
// draw the ship

if (ship.state==SHIP_STATE_ON)
{
ship.x = ship.varsF[SHIP_X_POS]+0.5;
ship.y = ship.varsF[SHIP_Y_POS]+0.5;
ship.curr_frame = 0;
Draw_BOB(&ship, lpddsback);
}}


// Test Star Input

void Init_Stars(void)
{
// this function initializes all the stars in the star field

int index; // looping variable

for (index=0; index < MAX_STARS; index++)
    {

    // select plane that star will be in


               stars[index].color = STAR_COLOR_2;
               stars[index].plane = STAR_PLANE_2;
               

     // set fields that aren't plane specific
     stars[index].x = rand()%SCREEN_WIDTH;   // change this latter to reflect clipping
     stars[index].y = rand()%SCREEN_HEIGHT;  // region

    } // end for index

} // end Init_Stars

/////////////////////////////////////////////////////////////////////////////

void Move_Stars(void)
{
// this function moves the star field, note that the star field is always
// in screen coordinates, otherwise, we would need thousands of stars to
// fill up the universe instead of 50!

int index,   // looping variable
    star_x,  // used as fast aliases to star position
    star_y,
    plane_0_dx,
    plane_0_dy,
    plane_1_dx,
    plane_1_dy,
    plane_2_dx,
    plane_2_dy;

// pre-compute plane translations
plane_0_dx =  250;
plane_0_dy =  250;

plane_1_dx =  100;
plane_1_dy =  100;

plane_2_dx =  0;
plane_2_dy =  0;

// move all the stars based on the motion of the player

for (index=0; index < MAX_STARS; index++)
    {
    // locally cache star position to speed up calculations
    star_x = stars[index].x-5;
    star_y = stars[index].y; 

    if (star_x >= SCREEN_WIDTH)
        star_x = star_x-SCREEN_WIDTH;
    else
    if (star_x < 0)
        star_x = SCREEN_WIDTH + star_x;


    // reset stars position in structure
    stars[index].x = star_x;
    stars[index].y = star_y;

    } // end for index

} // end Move_Stars

/////////////////////////////////////////////////////////////////////////////

void Draw_Stars(void)
{
// this function draws all the stars

// lock back surface
DDraw_Lock_Back_Surface();

// draw all the stars
for (int index=0; index < MAX_STARS; index++)
    {
    // draw stars 
    Draw_Pixel(stars[index].x,stars[index].y, stars[index].color,back_buffer, back_lpitch);

    } // end for index

// unlock the secondary surface
DDraw_Unlock_Back_Surface();

} // end Draw_Stars


void Init_Plasma(void)
{
// this function initializes and loads all the plasma 
// weapon pulses

// the plasma animations
static int plasma_anim_player[] = {0,1,2,3,4,5,6,7},
   		   plasma_anim_enemy[]  = {8,9,10,11,12,13,14,15};

// load the plasma imagery 
Load_Bitmap_File(&bitmap8bit, "ENERGY8.BMP");



// create the first bob
Create_BOB(&plasma[0],0,0,8,8,16,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM,
            DDSCAPS_SYSTEMMEMORY);

// load animation frames
for (int frame=0; frame < 16; frame++)
     Load_Frame_BOB(&plasma[0],&bitmap8bit,frame,frame%8,frame/8,BITMAP_EXTRACT_MODE_CELL);  

// set animation rate
Set_Anim_Speed_BOB(&plasma[0],1);

// load animations
Load_Animation_BOB(&plasma[0], PLASMA_ANIM_PLAYER, 8, plasma_anim_player);
Load_Animation_BOB(&plasma[0], PLASMA_ANIM_ENEMY, 8, plasma_anim_enemy);

// set state to off
plasma[0].state = PLASMA_STATE_OFF;

for (int pulse=1; pulse < MAX_PLASMA; pulse++)
    {
    memcpy(&plasma[pulse], &plasma[0], sizeof(BOB));
    } // end for pulse

// unload data infile
Unload_Bitmap_File(&bitmap8bit);



} // end Init_Plasma

/////////////////////////////////////////////////////////

void Reset_Plasma(void)
{
// this function resets all the plasma pulses
for (int pulse = 0; pulse < MAX_PLASMA; pulse++)
    {
    plasma[pulse].state = PLASMA_STATE_OFF;
    } // end for pulse

} // end Reset_Plasma

///////////////////////////////////////////////////////////

void Delete_Plasma(void)
{
// this function simply deletes all memory and surfaces
// related to the plasma pulses

for (int index=0; index < MAX_PLASMA; index++)
    Destroy_BOB(&plasma[index]);

} // end Delete_Plasma

///////////////////////////////////////////////////////////

///////////////////////////////////////////////////

void Draw_Plasma(void)
{
// this function draws all the plasma pulses

for (int index=0; index<MAX_PLASMA; index++)
    {
    // test if plasma pulse is in flight
    if (plasma[index].state == PLASMA_STATE_ON)
        {
        // transform to screen coords

        //plasma[index].x = plasma[index].varsI[INDEX_WORLD_X] - (plasma[index].width >> 1) - player_x + (SCREEN_WIDTH/2);
        //plasma[index].y = plasma[index].varsI[INDEX_WORLD_Y] - (plasma[index].height >> 1) - player_y + (SCREEN_HEIGHT/2);

		plasma[index].x = plasma[index].varsI[INDEX_WORLD_X]+20 ;
        plasma[index].y = plasma[index].varsI[INDEX_WORLD_Y] ;

        // draw the pulse
        Draw_BOB(&plasma[index],lpddsback);
         
        // animate the pulse
        Animate_BOB(&plasma[index]);

        } // end if

    } // end for index

} // end Draw_Plasma

///////////////////////////////////////////////////////////

 void Move_Plasma(void)
{
// this function moves all the plasma pulses and checks for
// collision with the rocks

for (int index=0; index < MAX_PLASMA; index++)
    {
    // test if plasma pulse is in flight
    if (plasma[index].state == PLASMA_STATE_ON)
        {
        // move the pulse 
        plasma[index].varsI[INDEX_WORLD_X]+=plasma[index].xv;
        //plasma[index].varsI[INDEX_WORLD_Y]+=plasma[index].yv;
          
        // test for boundaries
        if ( (++plasma[index].counter_1 > PLASMA_RANGE_1)              || 
                (plasma[index].varsI[INDEX_WORLD_X] > SCREEN_WIDTH)  || 
                (plasma[index].varsI[INDEX_WORLD_X] < 0 ) ||
	            (plasma[index].varsI[INDEX_WORLD_Y] > SCREEN_HEIGHT)  || 
                (plasma[index].varsI[INDEX_WORLD_Y] < 0 ) )
            {
            // kill the pulse
            plasma[index].state = PLASMA_STATE_OFF;
            plasma[index].counter_1 = 0;  }
 
 // test for collision with rocks
        for (int rock=0; rock < MAX_ROCKS-ROCK_OFFSET; rock++)
            {
            if (rocks[rock].state==ROCK_STATE_ON)
                {
                // test for collision 
                if (Collision_Test(plasma[index].varsI[INDEX_WORLD_X]-(plasma[index].width*.5), 
					               plasma[index].varsI[INDEX_WORLD_Y]-(plasma[index].height*.5), 
                                   plasma[index].width, plasma[index].height,
                                   rocks[rock].varsI[INDEX_WORLD_X]-(rocks[rock].width*.2), 
								   rocks[rock].varsI[INDEX_WORLD_Y]+(rocks[rock].height*.2),
                                   rocks[rock].width-20, rocks[rock].height-20))
                    {
                    // kill pulses and rock
					SCORE=SCORE+100;
					ROCKS_DESTROYED++;
					TOTAL_ROCKS_DESTROYED++;
					ROCKS_OUT--;
					FRAME_TIMER=0;
					rocks[rock].state=ROCK_STATE_OFF;
			
                    plasma[index].state = PLASMA_STATE_OFF;
                    plasma[index].counter_1 = 0;
					

					if((ROCKS_DESTROYED > 4 && LEVEL<5 && ROCKS_OUT<=0) || (ROCKS_DESTROYED > 10 && LEVEL>=5 && ROCKS_OUT<=0) )
					{	ROCKS_DESTROYED=0;
						Level_Increment();
					}
  
Start_Burst(plasma[index].varsI[INDEX_WORLD_X], plasma[index].varsI[INDEX_WORLD_Y], 
                                          68+rand()%12,54+rand()%10,
                                          rocks[rock].xv*.5, rocks[rock].yv*.5,BURST_ROCK);
                                        
                            

                          
 }}}}}}



void Fire_Plasma(int x,int y, int xv, int yv, int source=PLASMA_ANIM_PLAYER)
{
// this function fires a plasma pulse at the given starting
// position and velocity, of course, one must be free for 
// this to work

// scan for a pulse that is available



for (int index=0; index < MAX_PLASMA; index++)
    {
    // is this one available
      // test if plasma pulse is in flight
    if (plasma[index].state == PLASMA_STATE_OFF)
       {

		
       // start this one up, note the use of world coords
       plasma[index].varsI[INDEX_WORLD_X] = x;
       plasma[index].varsI[INDEX_WORLD_Y] = y;
       
       plasma[index].xv = xv;
	   plasma[index].yv = yv;
       plasma[index].curr_frame = 0;
       plasma[index].state =  PLASMA_STATE_ON;
       plasma[index].counter_1 = 0; // used to track distance

	   // set animation and typing info for collision engine
	   plasma[index].anim_state = source;
	   Set_Animation_BOB(&plasma[index], source);

	   if (DSound_Status_Sound(laser_id)==0)
              {
              DSound_Play(laser_id);
                        } 


       // later
       return;

       } // end if

    } // end for

} // end Fire_Plasma

void Init_Rocks(void)
{
// this function initializes and loads all the rocks 

int frame; // used as loop index

// create the large rock
Create_BOB(&rock_l,0,0,96,96,16,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

// load animation frames
for (frame=0; frame <= 15; frame++)
	{
	// load the rocks imagery 
    Pad_Name("ROCKL", "BMP", buffer, frame);
	Load_Bitmap_File(&bitmap8bit, buffer);		

    // load the actual .BMP
    Load_Frame_BOB(&rock_l,&bitmap8bit,frame,0,0,BITMAP_EXTRACT_MODE_ABS); 
	
	Set_Palette(bitmap8bit.palette);

    // unload data infile
    Unload_Bitmap_File(&bitmap8bit);

	} // end if

// set animation rate
Set_Anim_Speed_BOB(&rock_l,1+rand()%5);
Set_Vel_BOB(&rock_l, -4+rand()%8, 4+rand()%4);
Set_Pos_BOB(&rock_l, rand()%WINDOW_WIDTH, rand()%SCREEN_HEIGHT-30);

// set size of rock
rock_l.varsI[0] = ROCK_LARGE;
rock_l.varsI[1] = rock_sizes[rock_l.varsI[0]];

// set state to off
rock_l.state = ROCK_STATE_ON;

/*// create the medium rock
Create_BOB(&rock_m,0,0,56,56,16,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

// load animation frames
for (frame=0; frame <= 15; frame++)
	{
	// load the rocks imagery 
    Pad_Name("ROCKM", "BMP", buffer, frame);
	Load_Bitmap_File(&bitmap8bit, buffer);		

    // load the actual .BMP
    Load_Frame_BOB(&rock_m,&bitmap8bit,frame,0,0,BITMAP_EXTRACT_MODE_ABS);  

    // unload data infile
    Unload_Bitmap_File(&bitmap8bit);

	} // end if

// set animation rate
Set_Anim_Speed_BOB(&rock_m,1+rand()%5);
Set_Vel_BOB(&rock_m, -4+rand()%8, 4+rand()%4);
Set_Pos_BOB(&rock_m, rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT);

// set size of rock
rock_m.varsI[0] = ROCK_MEDIUM;
rock_m.varsI[1] = rock_sizes[rock_m.varsI[0]];

// set state to off
rock_m.state = ROCK_STATE_OFF;

// create the small rock
Create_BOB(&rock_s,0,0,32,32,16,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

// load animation frames
for (frame=0; frame <= 15; frame++)
	{
	// load the rocks imagery 
    Pad_Name("ROCKS", "BMP", buffer, frame);
	Load_Bitmap_File(&bitmap8bit, buffer);		

    // load the actual .BMP
    Load_Frame_BOB(&rock_s,&bitmap8bit,frame,0,0,BITMAP_EXTRACT_MODE_ABS);  

    // unload data infile
    Unload_Bitmap_File(&bitmap8bit);

	} // end if

// set animation rate
Set_Anim_Speed_BOB(&rock_s,1+rand()%5);
Set_Vel_BOB(&rock_s, -4+rand()%8, -4+rand()%8);
Set_Pos_BOB(&rock_s, rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT);

// set size of rock
rock_s.varsI[0] = ROCK_SMALL;
rock_s.varsI[1] = rock_sizes[rock_s.varsI[0]];

// set state to off
rock_s.state = ROCK_STATE_OFF;

  */

// make semi copies
for (int rock=0; rock < MAX_ROCKS; rock++)
    {
    memcpy(&rocks[rock], &rock_l, sizeof(BOB));

    // set animation rate
    Set_Anim_Speed_BOB(&rocks[rock],1+rand()%5);
    
    // set velocity
    Set_Vel_BOB(&rocks[rock], 1+rand()%4, 1+rand()%4);
    
    // set position
    Set_Pos_BOB(&rocks[rock],0,0);
    rocks[rock].varsI[INDEX_WORLD_X] = SCREEN_WIDTH;
    rocks[rock].varsI[INDEX_WORLD_Y] = RAND_RANGE(0, SCREEN_HEIGHT-30);    

    // set size of rock
    rocks[rock].varsI[0] = rand()%3;
    rocks[rock].varsI[1] = rock_sizes[rocks[rock].varsI[0]];

	rocks[rock].state=ROCK_STATE_ON;

    // based on size of rock switch in correct bitmaps and sizes
	switch(rocks[rock].varsI[1])
		{
		case ROCK_LARGE:		
			{
            // copy dd bitmap surfaces
            memcpy(rocks[rock].images,rock_l.images,sizeof(LPDIRECTDRAWSURFACE)*MAX_BOB_FRAMES);

			// set width and height (incase)
			rocks[rock].width  = rock_l.width;
			rocks[rock].height = rock_l.height;

			} break;

		/*case ROCK_MEDIUM:
			{
            // copy dd bitmap surfaces
            memcpy(rocks[rock].images,rock_m.images,sizeof(LPDIRECTDRAWSURFACE)*MAX_BOB_FRAMES);

			// set width and height (incase)
			rocks[rock].width  = rock_m.width;
			rocks[rock].height = rock_m.height;

			} break;

		case ROCK_SMALL:
			{
            // copy dd bitmap surfaces
            memcpy(rocks[rock].images,rock_s.images,sizeof(LPDIRECTDRAWSURFACE)*MAX_BOB_FRAMES);

			// set width and height (incase)
			rocks[rock].width  = rock_s.width;
			rocks[rock].height = rock_s.height;

			} break;*/
		
		default: break;

		} // end switch
   
    } // end for rock

} // end Init_Rocks

//////////////////////////////////////////////////////////////////////////////

void Refresh_Rocks(void)
{

	ROCKS_OUT=MAX_ROCKS-ROCK_OFFSET;

	for (int rock=0; rock < MAX_ROCKS-ROCK_OFFSET; rock++)
    {
	// set animation rate
    Set_Anim_Speed_BOB(&rocks[rock],1+rand()%5);
    
    // set velocity
    Set_Vel_BOB(&rocks[rock], 1+rand()%SPEED_OFFSET, 1+rand()%SPEED_OFFSET);
    
    // set position
    rocks[rock].varsI[INDEX_WORLD_X] = SCREEN_WIDTH;
    rocks[rock].varsI[INDEX_WORLD_Y] = RAND_RANGE(0, SCREEN_HEIGHT-30);    

	rocks[rock].state=ROCK_STATE_ON;}

}

void Draw_Rocks(void)
{
// this function draws all the rocks 

for (int index=0; index<MAX_ROCKS-ROCK_OFFSET; index++)
    {
	if (rocks[index].state==ROCK_STATE_ON)
	{	
        // transform to screen coords
        rocks[index].x = rocks[index].varsI[INDEX_WORLD_X];
        rocks[index].y = rocks[index].varsI[INDEX_WORLD_Y];
	
		// draw the rock
        Draw_BOB(&rocks[index],lpddsback);
         
        // animate the pulse
        Animate_BOB(&rocks[index]);
}

    } // end for index

} // end Draw_Rocks

///////////////////////////////////////////////////////////

void Move_Rocks(void)
{
// this function moves all the rocks pulses and checks for
// collision with the rocks

for (int index=0; index < MAX_ROCKS-ROCK_OFFSET; index++)
    {
    // test if rocks pulse is in flight

        // move the rock
        rocks[index].varsI[INDEX_WORLD_X]=rocks[index].varsI[INDEX_WORLD_X]-rocks[index].xv;
		
		// test if rock is out of universe, but persist object
        if (rocks[index].varsI[INDEX_WORLD_X] < 0)
		{
            rocks[index].varsI[INDEX_WORLD_X] = SCREEN_WIDTH;
			rocks[index].varsI[INDEX_WORLD_Y] = RAND_RANGE(0, SCREEN_HEIGHT-30);   

		}

		

		//sprintf(buffer, "Position = ( %f,%f ) use <RIGHT>, <LEFT> arrows to change position", rocks[index].varsI[INDEX_WORLD_X]-((rocks[index].width ) *.5),rocks[index].varsI[INDEX_WORLD_Y]+((rocks[index].height ) *.5) );
	//Draw_Text_GDI(buffer,10, 45,RGB(255,255,255), lpddsback);


		//Sleep(100);


		//Test for ship collision with rock.

		if ( Collision_Test(ship.varsF[SHIP_X_POS]-(ship.width*.5), ship.varsF[SHIP_Y_POS]+(ship.height*.5), 
                           ship.width, ship.height,
                           rocks[index].varsI[INDEX_WORLD_X]-((rocks[index].width ) *.2), 
						   rocks[index].varsI[INDEX_WORLD_Y]+((rocks[index].height ) *.2),
                           rocks[index].width-20, rocks[index].height-20) 
						   && ship.state==SHIP_STATE_ON && rocks[index].state==ROCK_STATE_ON)
        {
        // what size rock did we hit?

     
                   Start_Burst(rocks[index].varsI[INDEX_WORLD_X], rocks[index].varsI[INDEX_WORLD_Y], 
                               68+rand()%12,54+rand()%10,
                               rocks[index].xv*.5, rocks[index].yv*.5,BURST_PLAYER);
				   PLAYER_LIVES=PLAYER_LIVES-1;
				   FRAME_TIMER=0;
				   CRASH = 1;
				   ship.state=SHIP_STATE_OFF;
		}


		



		}}

///////////////////////////////////////////////////////////
void Draw_Bursts(void)
{
// this function draws all the bursts 
for (int index=0; index < MAX_BURSTS; index++)
    {
    // test if bursts pulse is in flight
    if (bursts[index].state == BURST_STATE_ON)
        {
		// transform bursts to screen space
		if (bursts[index].varsI[2]==BURST_PLAYER)
		{ 
		bursts[index].x = ship.varsF[SHIP_X_POS];
        bursts[index].y = ship.varsF[SHIP_Y_POS];
		}
		else if (bursts[index].varsI[2]==BURST_ROCK)
		{bursts[index].x=bursts[index].varsI[INDEX_WORLD_X];
		bursts[index].y=bursts[index].varsI[INDEX_WORLD_Y];}

		// is scaling needed
        if (bursts[index].varsI[0]!=bursts[index].width || 
            bursts[index].varsI[1]!=bursts[index].height)
            {
            // draw the burst scaled
            Draw_Scaled_BOB(&bursts[index],
                            bursts[index].varsI[0],bursts[index].varsI[1],
                            lpddsback);
            }
        else // draw normal
           Draw_BOB(&bursts[index],lpddsback);
         
        // animate the explosion
        Animate_BOB(&bursts[index]);

        } // end if

    } // end for index

} // end Draw_Bursts

///////////////////////////////////////////////////////////

void Start_Burst(int x, int y, int width, int height, int xv,int yv,int burst_type)
{

				
		//if (DSound_Status_Sound(explode_id)==0)
		//{
		DSound_Play(explode_id); // end if
		//}
// this function starts a burst up

// now test if it's time to add a new burst to the list

// scan for a burst to initialize
for (int index=0; index < MAX_BURSTS; index++)
    {
    // is this burst available?
    if (bursts[index].state == BURST_STATE_OFF)
       {
       // set animation rate
       Set_Anim_Speed_BOB(&bursts[index],1);
       bursts[index].curr_frame = 0;
    
       // set velocity
       Set_Vel_BOB(&bursts[index], xv,yv);
    
       // set position
       Set_Pos_BOB(&bursts[index], 0,0);
	   bursts[index].varsI[INDEX_WORLD_X] = x;
	   bursts[index].varsI[INDEX_WORLD_Y] = y;
    
       // set size of burst
       bursts[index].varsI[0] = width;
       bursts[index].varsI[1] = height;
	   bursts[index].varsI[2] = burst_type;

       // turn burst on
       bursts[index].state = BURST_STATE_ON;

       
       


       // start sound up

           
        
                

      
       return;

       } // end if
 
    } // end for index

} // end Start_Burst

///////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////

void Init_Bursts(void)
{
// this function initializes and loads all the bursts 

// load the bursts imagery 
Load_Bitmap_File(&bitmap8bit, "EXPL8.BMP");

// create the first bob
Create_BOB(&bursts[0],0,0,42,36,14,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

// load animation frames
for (int frame=0; frame < 14; frame++)
     Load_Frame_BOB(&bursts[0],&bitmap8bit,frame,frame%6,frame/6,BITMAP_EXTRACT_MODE_CELL);  

// set animation rate
Set_Anim_Speed_BOB(&bursts[0],1);

// set size of burst
bursts[0].varsI[0] = bursts[0].width;
bursts[0].varsI[1] = bursts[0].height;


// set state to off
bursts[0].state = BURST_STATE_OFF;

for (int burst=1; burst<MAX_BURSTS; burst++)
    {
    memcpy(&bursts[burst], &bursts[0], sizeof(BOB));
    } // end for burst

// unload data infile
Unload_Bitmap_File(&bitmap8bit);

} // end Init_Bursts

///////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////

void Reset_Bursts(void)
{
// this function resets all the bursts

for (int burst=0; burst < MAX_BURSTS; burst++)
    {
    // set state to off
    bursts[burst].state = BURST_STATE_OFF;
    } // end for burst


} // end Reset_Bursts


////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////

void Move_Bursts(void)
{
// this function moves all the bursts 

for (int index=0; index < MAX_BURSTS; index++)
    {
    // test if bursts pulse is in moving
    if (bursts[index].state == BURST_STATE_ON)
       {
       // move the burst
       // Move_BOB(&bursts[index]);
       bursts[index].varsI[INDEX_WORLD_X]+=bursts[index].xv;
       bursts[index].varsI[INDEX_WORLD_Y]+=bursts[index].yv;

        // test if burst is off screen or done with animation
        if (bursts[index].curr_frame >= bursts[index].num_frames-1) 
           {
           // kill burst and put back on available list
           bursts[index].state = BURST_STATE_OFF;
           } // end if
        
        } // end if

    } // end for index

} // end Move_Bursts

///////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////

void Do_Intro(int intro_index)
{
// the worlds simples intro

// clear out buffers
DDraw_Fill_Surface(lpddsback, 0);
DDraw_Fill_Surface(lpddsprimary, 0);

// draw in logo screen
if (intro_index==1)
Load_Bitmap_File(&bitmap8bit, "intro.bmp");

if (intro_index==2)
Load_Bitmap_File(&bitmap8bit, "intro2.bmp");

if (intro_index==3)
Load_Bitmap_File(&bitmap8bit, "intro3.bmp");

// set the palette to background image palette
Set_Palette(bitmap8bit.palette);

// copy the bitmap to primary buffer
DDraw_Lock_Primary_Surface();
Copy_Screen(bitmap8bit.buffer,primary_buffer, primary_lpitch, 0);
DDraw_Unlock_Primary_Surface();

// unload bitmap file
Unload_Bitmap_File(&bitmap8bit);

Sleep(5000);

// transition to black
Screen_Transitions(SCREEN_DARKNESS,NULL,0);

// clear out buffers
DDraw_Fill_Surface(lpddsback, 0);
DDraw_Fill_Surface(lpddsprimary, 0);

} // end Do_Intro

///////////////////////////////////////////////////////////

// WINMAIN ////////////////////////////////////////////////

int WINAPI WinMain(	HINSTANCE hinstance,
					HINSTANCE hprevinstance,
					LPSTR lpcmdline,
					int ncmdshow)
{
// this is the winmain function

WNDCLASS winclass;	// this will hold the class we create
HWND	 hwnd;		// generic window handle
MSG		 msg;		// generic message
HDC      hdc;       // generic dc
PAINTSTRUCT ps;     // generic paintstruct

// first fill in the window class stucture
winclass.style			= CS_DBLCLKS | CS_OWNDC | 
                          CS_HREDRAW | CS_VREDRAW;
winclass.lpfnWndProc	= WindowProc;
winclass.cbClsExtra		= 0;
winclass.cbWndExtra		= 0;
winclass.hInstance		= hinstance;
winclass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
winclass.hCursor		= LoadCursor(NULL, IDC_ARROW);
winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
winclass.lpszMenuName	= NULL; 
winclass.lpszClassName	= WINDOW_CLASS_NAME;

// register the window class
if (!RegisterClass(&winclass))
	return(0);

// create the window, note the use of WS_POPUP
if (!(hwnd = CreateWindow(WINDOW_CLASS_NAME, // class
						  "Collision Demo",	 // title
						  WS_POPUP | WS_VISIBLE,
					 	  0,0,	   // x,y
						  WINDOW_WIDTH,  // width
                          WINDOW_HEIGHT, // height
						  NULL,	   // handle to parent 
						  NULL,	   // handle to menu
						  hinstance,// instance
						  NULL)))	// creation parms
return(0);

// save the window handle and instance in a global
main_window_handle = hwnd;
main_instance      = hinstance;

// perform all game console specific initialization
Game_Init();

// enter main event loop
while(1)
	{
	if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{ 
		// test if this is a quit
        if (msg.message == WM_QUIT)
           break;
	
		// translate any accelerator keys
		TranslateMessage(&msg);

		// send the message to the window proc
		DispatchMessage(&msg);
		} // end if
    
    // main game processing goes here
    Game_Main();

	} // end while

// shutdown game and release all resources
Game_Shutdown();

// return to Windows like this
return(msg.wParam);

} // end WinMain

// T3D GAME PROGRAMMING CONSOLE FUNCTIONS ////////////////

int Game_Init(void *parms)
{
// this function is where you do all the initialization 
// for your game

int index; // looping varsIable

char filename[80]; // used to build up filenames

// seed random number generate
srand(Start_Clock());

// start up DirectDraw (replace the parms as you desire)
DDraw_Init(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP);

Do_Intro(1);
Do_Intro(2);
Do_Intro(3);


// load background image

Load_Bitmap_File(&bitmap8bit, "LAVA.BMP");
Create_Bitmap(&background_bmp,0,0,640,480);
Load_Image_Bitmap(&background_bmp, &bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Set_Palette(bitmap8bit.palette);
Unload_Bitmap_File(&bitmap8bit);

// hide the mouse
ShowCursor(FALSE);

// initialize directinput
DInput_Init();

// acquire the keyboard only
DInput_Init_Keyboard();

// initilize DirectSound/Music
DSound_Init();
DMusic_Init();

// load background sounds
engine_id = DSound_Load_WAV("engines.WAV");
laser_id = DSound_Load_WAV("lasertw.WAV");
explode_id = DSound_Load_WAV("cannon2.WAV");
background_id[0] = DMusic_Load_MIDI("MM1CUT.MID");
background_id[1] = DMusic_Load_MIDI("MM1ELEC.MID");
background_id[2] = DMusic_Load_MIDI("WOODMAN3.MID");
background_id[3] = DMusic_Load_MIDI("AIRMAN.MID");
background_id[4] = DMusic_Load_MIDI("FLASH3.MID");

DMusic_Play(background_id[(RAND_RANGE(0,4))]);

// start the sounds
//DSound_Play(engine_id, DSBPLAY_LOOPING);

// set clipping rectangle to screen extents so objects dont
// mess up at edges
RECT screen_rect = {0,0,screen_width,screen_height};
lpddclipper = DDraw_Attach_Clipper(lpddsback,1,&screen_rect);

// initialize all the plasma
Init_Plasma();

// initialize all the explosions
Init_Bursts();

// initialize all the stars
Init_Stars();

// initialize all the rocks
Init_Rocks();

//initialize the ship
Init_Ship();

ROCKS_OUT=MAX_ROCKS-ROCK_OFFSET;



// return success
return(1);

} // end Game_Init

///////////////////////////////////////////////////////////

int Game_Shutdown(void *parms)
{
// this function is where you shutdown your game and
// release all resources that you allocated

// shut everything down

// kill all the bobs
Destroy_BOB(&ship);

// shutdown directdraw last
DDraw_Shutdown();

// now directsound
DSound_Stop_All_Sounds();
DSound_Shutdown();

// shut down directinput
DInput_Shutdown();

// return success
return(1);

} // end Game_Shutdown

//////////////////////////////////////////////////////////

int Game_Main(void *parms)
{
// this is the workhorse of your game it will be called
// continuously in real-time this is like main() in C
// all the calls for you game go here!

int index; // looping var

//static int delay = 1; // initial delay per frame

// start the timing clock
DWORD start_time = Start_Clock();

// lock back buffer and copy background into it
DDraw_Lock_Back_Surface();

// draw background
Draw_Bitmap(&background_bmp, back_buffer, back_lpitch,0);

// unlock back surface
DDraw_Unlock_Back_Surface();

// draw the stars

Move_Stars();
Draw_Stars();

Move_Plasma();
Draw_Plasma();

Move_Rocks();
Draw_Rocks();


//Set_Palette(game_palette);

Move_Bursts();
Draw_Bursts();

if ( PLAYER_LIVES == 0 ) 
{
Game_Over();

sprintf(buffer, "GAME OVER");
Draw_Text_GDI(buffer,230, 210,RGB(255,0,0), lpddsback);
sprintf(buffer, "SCORE: %i", SCORE );
Draw_Text_GDI(buffer,230, 225,RGB(255,255,255), lpddsback);
sprintf(buffer, "TOTAL ROCKS DESTROYED: %i", TOTAL_ROCKS_DESTROYED );
Draw_Text_GDI(buffer,230, 240,RGB(255,255,255), lpddsback);
sprintf(buffer, "PRESS ESC TO QUIT OR PRESS F2 TO RESTART");
Draw_Text_GDI(buffer,230, 255,RGB(255,0,0), lpddsback);
}

if (CRASH != 1)
{Draw_Ship();}

if (CRASH == 1 && FRAME_TIMER > 100 && PLAYER_LIVES!=0)
{// use element 0,1 to hold float accurate position
ship.state=SHIP_STATE_ON;
ship.varsF[SHIP_X_POS] = 0;
ship.varsF[SHIP_Y_POS] = screen_height/2;
Draw_Ship();
CRASH=0;}

if (ROCKS_OUT <= 0 && FRAME_TIMER > 100)
{
Refresh_Rocks();

}

// read keyboard
DInput_Read_Keyboard();



// Move Ship.  Stop at Screen Edge.
if (keyboard_state[DIK_RIGHT] && ship.varsF[SHIP_X_POS] < SCREEN_WIDTH && ship.state==SHIP_STATE_ON ) 
	ship.varsF[SHIP_X_POS]+=Speed;
 
if (keyboard_state[DIK_LEFT] && ship.varsF[SHIP_X_POS] > 0 && ship.state==SHIP_STATE_ON) 
	ship.varsF[SHIP_X_POS]=ship.varsF[SHIP_X_POS]-Speed;

if (keyboard_state[DIK_UP] && ship.varsF[SHIP_Y_POS] > 0 && ship.state==SHIP_STATE_ON) 
	ship.varsF[SHIP_Y_POS]=ship.varsF[SHIP_Y_POS]-Speed;

if (keyboard_state[DIK_DOWN] && ship.varsF[SHIP_Y_POS] < SCREEN_HEIGHT && ship.state==SHIP_STATE_ON) 
	ship.varsF[SHIP_Y_POS]=ship.varsF[SHIP_Y_POS]+Speed;

if (keyboard_state[DIK_LSHIFT] && Speed==3 && !speed_lock)
{Speed=6;
speed_lock=1;}
else if (keyboard_state[DIK_LSHIFT] && Speed==6 && !speed_lock)
{Speed=3;
speed_lock=1;}

if ( !keyboard_state[DIK_LSHIFT] )
{speed_lock=0;}


if (keyboard_state[DIK_SPACE] && ship.state==SHIP_STATE_ON && FRAME_TIMER%7==0)
{
player_x=ship.varsF[SHIP_X_POS];
player_y=ship.varsF[SHIP_Y_POS];

Fire_Plasma(player_x,player_y,6,6, PLASMA_ANIM_PLAYER);
}



sprintf(buffer, "Position = ( %f,%f ) ", ship.varsF[SHIP_X_POS],ship.varsF[SHIP_Y_POS] );
Draw_Text_GDI(buffer,10, 25,RGB(255,255,255), lpddsback);

sprintf(buffer, "Score: %i Player Lives: %i LEVEL: %i " ,SCORE,PLAYER_LIVES,LEVEL );
Draw_Text_GDI(buffer,10, 40,RGB(255,255,255), lpddsback);

// flip the surfaces
DDraw_Flip();

// compute dt
float dt = Get_Clock() - start_time; // in milliseconds

// test for off screen
if (ship.varsF[SHIP_X_POS] > screen_width+ship.width)
   ship.varsF[SHIP_X_POS] = -ship.width;

//////////////////////////////////////////////////////

// Check for game restart
if (keyboard_state[DIK_F2])
{Game_Restart();}

// check of user is trying to exit
if (KEY_DOWN(VK_ESCAPE) || keyboard_state[DIK_ESCAPE])
    {
    PostMessage(main_window_handle, WM_DESTROY,0,0);

    // stop all sounds
    DSound_Stop_All_Sounds();

    // do a screen transition
    Screen_Transitions(SCREEN_DARKNESS,NULL,0);
    } // end if

FRAME_TIMER=FRAME_TIMER+1;



// return success
return(1);

} // end Game_Main

//////////////////////////////////////////////////////////